<?php

declare(strict_types=1);

namespace Rowlinson\Api;

use Rowlinson\Api\Collections\PaginatedCollection;
use Rowlinson\Api\Requests\Order\PlaceOrderRequest;
use Rowlinson\Api\Responses\Order\OrderResponse;
use Rowlinson\Api\Responses\Order\OrdersResponse;

class OrderClient extends AbstractClient
{
    /**
     * Using the current basket for the user, place an order with the supplied details
     *
     * @param array<string,string>|null $newDeliveryLocation
     */
    public function placeOrder(
        ?string $deliveryLocationId,
        ?string $customerReference,
        ?string $deliveryInstructions,
        ?array $newDeliveryLocation
    ): void {
        $this->post(
            'basket/confirm',
            new PlaceOrderRequest($deliveryLocationId, $customerReference, $deliveryInstructions, $newDeliveryLocation)
        );
    }

    /**
     * Get the orders for the organisation of the currently logged in user
     *
     * @return PaginatedCollection<OrdersResponse>
     */
    public function getOrders(int $page = 1): PaginatedCollection
    {
        $response = $this->get('orders?' . http_build_query([
            'page' => $page,
        ]));

        $orderResponses = $this->serializer->deserialize(
            (string)$response->getBody(),
            'array<' . OrdersResponse::class . '>', // @phpstan-ignore-line
            'json'
        );

        return new PaginatedCollection(
            $orderResponses,
            $page,
            $response->getPaginationPerPage(),
            $response->getPaginationTotalItems()
        );
    }

    /**
     * Get the details of an individual order
     */
    public function getOrder(string $orderNumber): ?OrderResponse
    {
        $response = $this->get('orders/' . $orderNumber);

        return $this->serializer->deserialize((string)$response->getBody(), OrderResponse::class, 'json');
    }
}
